package Logaholic::Preparse;

use strict;
use warnings;

use Cwd qw(abs_path);
use Digest::MD5 qw(md5_hex);
use File::Basename qw( dirname );
use FindBin;
use Geo::IPfree;
use Logaholic::IO::File    ();
use Logaholic::Log::Domlog ();
use URI;
use URI::QueryParam;

my $expire_time = 1200;

sub new {
    my ( $class, $schema_version ) = @_;

    $schema_version ||= '';

    my $geo = Geo::IPfree->new();
    $geo->Faster();
    my $properties = {
        'profile_schema_version' => $schema_version,
        'profile_config_data'    => {},
        'profile_config_file'    => '',
        'handles'                => {
            'inputfile' => {
                'filename' => '',
                'fh'       => undef,
                'mode'     => Logaholic::IO::File::MODE_READ,
            },
            'outputfile' => {
                'filename' => '',
                'fh'       => undef,
                'mode'     => Logaholic::IO::File::MODE_WRITE,
            },
        },
        'sessioncounter' => 0,
        'total_hits'     => 0,
        'total_mb'       => 0,
        'total_bytes'    => {},
        'visitors'       => {},
        'geo'            => $geo 
    };
    return bless $properties, $class;
}

# This assumes the schema are relative to this file
sub _schema_directory {
    my $module_path = $INC{'Logaholic/Preparse.pm'};
    return abs_path( dirname($module_path) . '/../../schema' );
}

sub assign_handles {
    my ( $self, $profile_config_data ) = @_;
    $self->{'handles'}->{'inputfile'}->{'filename'} =
      $profile_config_data->{'inputfile'};
    $self->{'handles'}->{'outputfile'}->{'filename'} =
      $profile_config_data->{'outputfile'};
    return $self;
}

# get an aliased file handle object
sub getHandle {
    my ( $self, $alias ) = @_;
    $alias ||= '';
    if ( !exists( $self->{'handles'}->{$alias} ) ) {
        die("Unknown file handle alias: $alias");
    }

    if ( $self->{'handles'}->{$alias}->{'fh'} ) {
        return $self->{'handles'}->{$alias}->{'fh'};
    }

    if ( !$self->{'handles'}->{$alias}->{'filename'} ) {
        die("Alias '$alias' does not have an associate file");
    }
    my $file = $self->{'handles'}->{$alias}->{'filename'};
    my $mode = $self->{'handles'}->{$alias}->{'mode'};
    my $fh   = Logaholic::IO::File->new( $file, $mode );
    if ( !$fh ) {
        die(
"Could not retreive a file handle for file '$file' with mode '$mode'"
        );
    }
    $self->{'handles'}->{$alias}->{'fh'} = $fh;
    return $fh;
}

sub get_config_value {
    my ( $self, $key ) = @_;
    if ( exists( $self->{'profile_config_data'}->{$key} ) ) {
        return $self->{'profile_config_data'}->{$key};
    }
    return '';
}

sub set_config_value {
    my ( $self, $key, $value ) = @_;
    return $self->{'profile_config_data'}->{$key} = $value;
}

sub load_config_from_filename {
    my ( $self, $config_file ) = @_;

    $self->{'profile_config_data'} = $self->get_profile_config($config_file);

    $self->assign_handles( $self->{'profile_config_data'} );

    if ( !$self->{'profile_config_data'}->{'expire_time'} ) {
        $self->{'profile_config_data'}->{'expire_time'} = $expire_time;
    }
	

    $self->{'profile_config_file'} = $config_file;

    return $self;
}

sub get_config_schema {
    my ( $self, $profile_schema_version ) = @_;
    my $schema         = {};
    my $app_schema_dir = _schema_directory();
    my $schema_file =
        $app_schema_dir
      . '/profile_config.'
      . $profile_schema_version
      . '.schema';
    open( my $schema_fh, '<', $schema_file )
      || die( 'Cannot open profile schema '
          . $profile_schema_version . ':'
          . $schema_file );
    while ( my $line = readline($schema_fh) ) {
        chomp $line;
        if ($line) {
            $schema->{$line} = '';
        }
    }
    close($schema_fh);
    return $schema;
}

sub get_profile_config {
    my ( $self, $config_file ) = @_;

    my $profile_config =
      $self->get_config_schema( $self->{'profile_schema_version'} );
    open( my $config_fh, '<', $config_file ) or die $!;

    # Loop through the config file, and create variables for each line
    while ( my $line = readline($config_fh) ) {
        chomp $line;
        if ($line) {
            my ( $key, $value ) = split( '=', $line, 2 );
            chomp $key;
            next if !$key;
            $value ||= '';
            chomp $value;
            $profile_config->{$key} = $value;
        }
    }
    close($config_fh);
    return $profile_config;
}

#########################################
## Actions
#########################################

sub update_statistics {
    my ( $self, $line ) = @_;

    $line->{'visitorid'} =$self->get_visitorid($line);
    $self->_update_bandwidth($line);

    if ( !$self->{'total_hits'} ) {
        $self->{'total_hits'} = 0;
    }
    ++$self->{'total_hits'};

    if ( !$self->{'total_mb'} ) {
        $self->{'total_mb'} = 0;
    }
    $self->{'total_mb'} += ( $line->{'bytes'} / ( 1024 * 1024 ) );

    $self->track_visit($line);
    $line->{'sessioncounter'} = $self->{'sessioncounter'};

    return $line;
}

# Parse URL Parameters (url params, urlparams), updating $line
sub parse_and_update_url_and_params {
    my ( $self, $line ) = @_;
    return if ( ref $line ne 'HASH' || !$line->{'url'} );

    my $dynamic_pages_filter = $self->get_dynamic_pages();
    ( $line->{'url'}, $line->{'urlparams'} ) =
      $self->_munge_uri_against_filterhash( $line->{'url'},
        $dynamic_pages_filter );
        $line->{'urlparams'} ||= '';
    if ($line->{'urlparams'} ) {
        if ( $line->{'urlparams'} !~ /^\?/ ) {
            $line->{'urlparams'} = '?' . $line->{'urlparams'};
        }
        $line->{'urlparams'} =~ tr/+/ /;
    }

    return $line;
}

sub _line_validation_points {
    my ($self) = @_;
    if (!$self->{'line_validation_points'}) {
        $self->{'line_validation_points'}->{'skipips'}   = $self->get_config_value('skipips');
        $self->{'line_validation_points'}->{'skipfiles'} = $self->get_config_value('skipfiles');
        
        while ( my ($key, $value) = each (%{$self->{'line_validation_points'}} )) {
            $self->{'line_validation_points'}->{$key} = qr/($value)/i if $value;
        }
        
        $self->{'line_validation_points'}->{'skiptime'}  = $self->get_config_value('skiptime') || 0;
        $self->{'line_validation_points'}->{'force'}     = $self->get_config_value('force') || 0;
    }

    return $self->{'line_validation_points'};
}
sub is_valid_line {
    my ( $self, $line ) = @_;

    return if ( ref $line ne 'HASH' || !$line->{'host'} || !$line->{'url'} );

    my $host      = $line->{'host'};
    my $url       = $line->{'url'};
    my $gmt = $line->{'gmt'} || 0;
    my $points = $self->_line_validation_points();

#tidyoff
# treat line as invalid if line has:
#  - no host or url
#  - host is listed in skipip profile config
#  - url is listed in skipfiles profile config
#  - no force was requested AND a skiptime further in future than current log line

    if (   ( $points->{'skipips'}   && $host =~ $points->{'skipips'} )
        || ( $points->{'skipfiles'} && $url =~ $points->{'skipfiles'} )
        || ( ! $points->{'force'}   && $points->{'skiptime'} && ( $gmt <=  $points->{'skiptime'} ) ) )
    {
        return;
    }

    #tidyon

    return 1;
}

sub add_refinements_to_line_data {
    my ( $self, $line ) = @_;

    $self->parse_and_update_url_and_params($line);
    $self->parse_and_update_keywords_and_refparams($line);

    $self->adjust_from_gmt_to_profile_timezone($line);
    
    return $line;
}

# This could be just as slow as the look up...once several lines of log
#  have been parsed and the hash grows
sub add_country_code {
    my ( $self, $line ) = @_;
    if (!exists($self->{'visitor_country'}->{$line->{'visitorid'}}) ) {
        $self->{'visitor_country'}->{$line->{'visitorid'}} = '';
        if ( $self->{'geo'} ) {
            $self->{'visitor_country'}->{$line->{'visitorid'}} =
              ( $self->{'geo'}->LookUp( $line->{'host'} ) )[0] || '';
        }
    }
    $line->{'country'} = $self->{'visitor_country'}->{$line->{'visitorid'}};
    return $line;
}

# Parse Referrer Parameters (refparams) and keywords
# NOTE: there was a bug previously in the equivadomain/confdomain which
# within the referrer string (due to an unachored replacement that would
# munge refparams with values of '://$equivdomain' instead of just the
# referrer itself). the result would be the refer would be treated as one
# originating from equivdomain/confdomain instead of the real referrer
# More specifically, if the real refer has a param that matches one that
# would normally be filter for confdomain, it could be lost or not
# removed properly (in equivdomain/confdomain or other refer domains)
#
# This as been resolved: refers are treated as refers; query params are
# filtered based solely on the refer domain.

sub parse_and_update_keywords_and_refparams {
    my ( $self, $line ) = @_;

    $line->{'keywords'}  = '';
    $line->{'refparams'} = '';
    if ( !$line->{'referrer'} ) {
        return $line;
    }

    my $keywords  = '';
    my $referrer  = $line->{'referrer'};
    my $refparams = '';
    my $authority = '';
    
    # combine all user profiles into a list and make a regex
    my $confdomain = $self->get_config_value('confdomain');
    my $user_domains =
      join( ',', ( $confdomain, $self->get_config_value('equivdomains') ) );
    my $domains_qr = Logaholic::Log::Domlog::domains_to_qr($user_domains);

    if ( $referrer =~ /\?/ ) {
        # get basic in case we have no specific filtering later
        # NOTE: the orig code assigned uri path to $referrer, but conditionally
        #  added the params back to it?  I'm leaving them separate, unconditionally
        my ($referrer_path, $uri_obj);
        ( $referrer_path, $refparams, $authority, $uri_obj ) =
          $self->_munge_uri_against_filterhash( $referrer, {} );

        #####
        ## Filter for different authorities, namely:
        # - user's
        # - google's
        # - yahoo's
        if ( !$authority ) {

            # weird for a referrer
            return $line;
        }

        # NOTE: the original code reads/operates in a rather ambiguous fashion
        #  by not limiting the filter of urlparams to user's domains, so
        #  it's possible that index.php urlparams meant for my site might
        #  be used on your site, expunge important data.  In this authored
        #  code, we honor the intent of the UI where the user adds those
        #  paths & urlparams
        if ( $authority =~ $domains_qr ) {
            # it was a referrer from site(s) belonging to user
            my $dynamic_pages_filter = $self->get_dynamic_pages();
            ( $referrer_path, $refparams, $authority, $uri_obj ) =
              $self->_munge_uri_against_filterhash( $referrer,
                $dynamic_pages_filter );

            # we fudge the refer to confdomain because other routines in the
            #  app will assume the difference is trivial
            $authority = $confdomain;
        }
        elsif ( $authority =~ m/^([a-z]*\.)?google\.[a-z]{2,3}/ ) {

            # google keyword detection

            $keywords = $self->get_google_keywords($uri_obj) || '';

            # no keywords?
            if ( $keywords eq '' ) {
                $keywords = "(not provided)";
            }

            #googleparams
            $refparams = $self->refine_google_params($uri_obj);

        }
        elsif ( $authority =~ m/^([a-z]*\.)?search.yahoo\.[a-z]{2,3}/ )
        {    # yahoo
            if ( $refparams =~ m/(?:^(?:[\?]?p\=)|(?:\&p\=))([^\&]+)/ ) {
                $keywords = $1 if $1;
            }
        }
        
        if ( $refparams =~ m/(?:^(?:[\?]?q\=)|(?:\&q\=))([^\&]+)/ ) {
            # generic keyword used by many

            $keywords = $1 if ($1 && (!$keywords || $keywords eq '(not provided)') );
        }
        
        $referrer  = $uri_obj->scheme() . '://' . $authority . $referrer_path;
        $line->{'referrer'} = $referrer;
    }
    else {
        my $uri_obj = URI->new($referrer);
        my $authority = $uri_obj->authority();
        
        if ($authority && $authority =~ $domains_qr) {
            $uri_obj->authority($confdomain);
        }
        
        $line->{'referrer'} = $uri_obj->canonical();
    }
    
    $line->{'keywords'} =
      ($keywords) ? lc( Logaholic::Log::Domlog::URLDecode($keywords) ) : '';
    $line->{'refparams'} =
      ($refparams) ? Logaholic::Log::Domlog::URLDecode($refparams) : '';
      $line->{'refparams'} = '?' . $line->{'refparams'} if ( $line->{'refparams'} && $line->{'refparams'} !~ /^\?/);
    return $line;
}

#########################################
## Misc
#########################################

sub enable_profile_timezone_warping {
	my ($self) = @_;
	my $profile_timezone = $self->get_config_value('timezone_offset');
	$self->{'warp_seconds'} = 0;
	if ($profile_timezone) {
		$profile_timezone =~ m/^([\+-])?(\d\d)(\d\d)/;
        if($1 eq '+' || $1 eq '-') {
        	$self->{'warp_op'} = $1;
        	$self->{'warp_seconds'} =  ( $2 * 3600 ) || 0;
        	if ($3) {
        		$self->{'warp_seconds'} += ($3 * 60);
        	}
        }
	}
	elsif ($profile_timezone = $self->get_config_value('timezone')) {
		# will have to use a dependency (that can handle zone strings)
		require DateTime;
		require DateTime::TimeZone;
		if (!exists($INC{'DateTime.pm'}) || !exists($INC{'DateTime/TimeZone.pm'})) {
			# DateTime not available
			return;
		}
		my $utc = DateTime->now('time_zone' => '0000');
		my $tz = DateTime::TimeZone->new( 'name' => $profile_timezone );
		my $offset = $tz->offset_for_datetime($utc);
		$offset =~ m/^([\+-])?(\d+)/;
        if ($2) {
            $self->{'warp_op'} = $1 || '+';
            $self->{'warp_seconds'} =  $2;
        }
	}

	return 1;
}

# this move the timestamp to a profile specific timezone
sub adjust_from_gmt_to_profile_timezone {
	my ($self, $line) = @_;
	return $line if ! $self->{'warp_seconds'};

	if ( $self->{'warp_op'} eq '+') {
        $line->{'gmt'} += $self->{'warp_seconds'};
    }
    else {
        $line->{'gmt'} -= $self->{'warp_seconds'};
    }

	return $line;
}

sub get_google_keywords {
    my ($self, $uri_obj) = @_;
    my $keywords;
    my @keyworddetectors = (
        'as_q',          'as_epq', 'as_oq', 'as_eq',
        'as_sitesearch', 'as_rq', 'as_lq'
    );
    my $strip_filter = $self->_get_google_keyword_filter();
    my $full_query = $uri_obj->query_form_hash();
    my $new_params =
              Logaholic::Preparse::apply_strip_filter_to_params_hash(
                $strip_filter, $full_query );

    # Despite $keywords being plural, we only expect one keyworddetector
    #  to actually be populated

    foreach my $value ( values %{ $new_params } ) {
        if ($value) {
            $keywords = $value;
            last;
        }
    }
    return $keywords;

}
sub refine_google_params {
    my ($self, $uri_obj) = @_;
    my $params_to_filter = $uri_obj->query_form_hash();
    my $strip_filter = $self->_get_google_strip_filter();
    my $new_params =
              Logaholic::Preparse::apply_strip_filter_to_params_hash(
                $strip_filter, $params_to_filter );
    $uri_obj->query_form_hash($new_params);
    return $uri_obj->query();
}

sub get_dynamic_pages {
    my ($self) = @_;
    if ( !$self->{'dynamic_pages'} ) {
        $self->{'dynamic_pages'} = {};
        my $importantparams = $self->get_config_value('importantparams');
        if ($importantparams) {
            $importantparams =~ s/\s//g;
            my @tmp_important = split( ',', $importantparams );

            foreach my $important (@tmp_important) {
                my ( $page, $value ) = split( "::", $important, 2 );
                $self->{'dynamic_pages'}->{$page} = [$value] if $page;
            }
        }
    }
    return $self->{'dynamic_pages'};
}

# store one line's worth of visit data
sub track_visit {
    my ( $self, $line ) = @_;
    my ( $host, $url, $referrer, $agent, $gmt ) = (
        $line->{'host'},  $line->{'url'}, $line->{'referrer'},
        $line->{'agent'}, $line->{'gmt'}
    );

    if ( exists( $self->{'visitors'}->{$host} ) ) {

        # there may be an 'active' visit/session happening for the host
        my $elapsed = $gmt - $self->{'visitors'}->{$host}->{'last_seen'};
        $elapsed =
          ( $elapsed < 0 ) ? $expire_time : $elapsed;  #protect against timewarp
        if ( $elapsed < $expire_time ) {

            # this visit has not expired, so add to existing record
            return $self->add_to_visit( $host, $gmt, $elapsed );
        }
    }

    # there is no visit currently "working" for this host
    #  OR
    # this visit has expired, so start a new one

    return $self->new_visit( $host, $gmt );
}

# record an entry for an access line that has been
# determined to represent a new visit (either because
# this is the first time this host has been seen,
#or because the host's previous visit has expired).
sub new_visit {
    my ( $self, $host, $gmt ) = @_;

    $self->{'sessioncounter'} ||= $self->get_config_value('sessioncounter');
    ++$self->{'sessioncounter'};
    $self->{'visitors'}->{$host}->{'last_seen'} = $gmt;
    return 1;
}

# append to an existing visit recond because it has been
# determinded that the current line contains more data to
# be added to a currently "working" visit
sub add_to_visit {
    my ( $self, $host, $gmt, $elapsed ) = @_;
    $self->{'visitors'}->{$host}->{'last_seen'} = $gmt;
    return 1;
}

sub _update_bandwidth {
    my ( $self, $line ) = @_;

# Create a key for bandwidth hash (assoc array)
# Because perl starts month counts at 0, we want to add 1 to the month key, so bandwidth has the right month
    my $mon_key = $line->{'datetime'}->{'mon'} + 1;

    # TODO: not sure why thisis here; the mon should always be 0-11 as defined
    #  by the lookup hash
    my $yr = $line->{'datetime'}->{'yr'};
    if ( $mon_key == 13 ) {
        $mon_key = 1;
        $yr      = $yr + 1;
    }

    $mon_key = ( length($mon_key) < 2 ) ? '0' . $mon_key : $mon_key;

    my $date_key =
        $yr . '-'
      . $mon_key . '-'
      . $line->{'datetime'}->{'day'};

    # Count the bytes per day
    if ( !$self->{'total_bytes'}->{$date_key} ) {
        $self->{'total_bytes'}->{$date_key} = 0;
    }
    $self->{'total_bytes'}->{$date_key} += $line->{'bytes'};
    return 1;
}

sub _get_strip_filter {
    my ($self) = @_;
    if ( !$self->{'strip_url_param_filter'} ) {
        my $urlparamfilter = $self->get_config_value('urlparamfilter') || '';
        $urlparamfilter =~ s/[\n\s]+//g;
        my @params = split( ',', $urlparamfilter );
        my $mode = lc( $self->get_config_value('urlparamfiltermode') ) || '';
        $self->{'strip_url_param_filter'} = {
            'params' => \@params,
            'mode'   => $mode || 'exclude',
        };
    }
    return $self->{'strip_url_param_filter'};
}

sub _get_google_strip_filter {
    my ($self) = @_;
    if ( !$self->{'google_strip_url_param_filter'} ) {
        my $googleparams = $self->get_config_value('googleparams') || '';
        $googleparams =~ s/[\n\s]+//g;
        my @params = split( /\|/, $googleparams );
        $self->{'google_strip_url_param_filter'} = {
            'params' => \@params,
            'mode'   => 'include',
        };
    }
    return $self->{'google_strip_url_param_filter'};
}

sub  _get_google_keyword_filter {
    my ($self) = @_;
    if ( !$self->{'google_strip_url_keyword_filter'} ) {
        my @keyworddetectors = (
            'as_q',          'as_epq', 'as_oq', 'as_eq',
            'as_sitesearch', 'as_rq', 'as_lq'
        );
        $self->{'google_strip_url_keyword_filter'} = {
            'params' => \@keyworddetectors,
            'mode'   => 'include',
        };
    }
    return $self->{'google_strip_url_keyword_filter'};
}

sub apply_strip_filter_to_params_hash {
    my ( $strip_filter, $url_params_and_values ) = @_;

    my $include = ( $strip_filter->{'mode'} eq 'include' ) ? 1 : 0;
    my $new_params = ($include) ? {} : \%{$url_params_and_values};

    foreach my $key ( @{ $strip_filter->{'params'} } ) {
        if ( exists( $url_params_and_values->{$key} ) ) {
            if ($include) {
                $new_params->{$key} = $url_params_and_values->{$key};
            }
            else {
                delete( $new_params->{$key} );
            }
        }
    }

    return $new_params;
}

# This subroutine takes
# 1) a URI
# 2) filter hash
# 3) regex of authority domains
#
# The return value is the URI's path, query params, and fqdn
#  - this specific values will be gathered/manipulated as follows:
#
# First, the URI is broken into path, query params, and domain
# A) if a regext of authority domains is passed, then the domain of the URI is
#    tested against it:
#   - if there is no match (or no domain was included in the URI), the sub
#     returns it's output without further analysis
# B) if filter hash is provided it will be treated as a 'map' of URI paths and
#    important query params that for all intents and purposes, should be
#    married to the path. (this helps for apps that don't support URL rewrite,
#    ie., index.php?page_id=blog)
# C) if the profile is set to include/exclude query params, then and (remaining)
#    params are filter against those (they are global, and not specific to any
#    particular path or domain).
#
# SOMEDAY: break this subroutine into smaller/more specific subs. The main
#  reason it currently is not is that it allows both request URIs and referrer
#  URIs to be passed it (keep code ops to a minimum)...but really, this sub is
#  too complex (as it the orignal code it was ported from).
sub _munge_uri_against_filterhash {
    my ( $self, $uri, $filterhash, $domain_authority ) = @_;

    $uri ||= '?';
    if ( $uri =~ m/^\:\/\// ) {

        # add a dummy schema
        $uri = 'http' . $uri;
    }
    my $uri_obj = URI->new($uri);
    my $path    = $uri_obj->path();

    # if the domain_authority is provided, it needs to be a qr//
    # NOTE: if it is provided, any provide $uri without authority will NOT be
    #  filtered!
    my $uri_authority = $uri_obj->authority() || '';
    if ( ref $domain_authority eq 'Regexp' ) {
        if (   ( !$uri_authority )
            || ( $uri_authority !~ $domain_authority ) )
        {
            #edge case for &amp;=
            my $params = $uri_obj->query_form_hash();
            if (exists($params->{''})) {
                delete $params->{''};
                delete $params->{'amp'} if (exists($params->{'amp'}));
                $uri_obj->query_form_hash($params);
            }
            
            return ( $path, $uri_obj->query(), $uri_authority, $uri_obj );
        }
    }

    my $params = $uri_obj->query_form_hash();
    my $query_params_str;
    if ( keys %{$params} ) {
        #edge case for &amp;=
        if (exists($params->{''})) {
            delete $params->{''};
            delete $params->{'amp'} if (exists($params->{'amp'}));
        }
        my $new_params;
        if ( defined $filterhash->{$path}
            && ref $filterhash->{$path} eq 'ARRAY' )
        {
            my @append_to_path;
            foreach my $important_param ( @{ $filterhash->{$path} } ) {
                if (defined($params->{$important_param})) {
                    push( @append_to_path,
                        $important_param . "=" . $params->{$important_param} );
                    delete( $params->{$important_param} );
                }
            }
            $path .= '?' . join( '&', @append_to_path );
            $new_params = $params;

        }

        my $strip_filter = $self->_get_strip_filter();
        my $params_to_filter = ( keys %{$new_params} ) ? $new_params : $params;
        if ( keys %{$params_to_filter} ) {
            $new_params =
              Logaholic::Preparse::apply_strip_filter_to_params_hash(
                $strip_filter, $params_to_filter );
        }

        if ( defined($new_params) ) {
            if ( keys %{$new_params} ) {
                $uri_obj->query_form_hash($new_params);
                $query_params_str = $uri_obj->query();
            }
            else {
                $query_params_str = '';
            }
        }
    }
    $query_params_str =
      ( defined($query_params_str) ) ? $query_params_str : $uri_obj->query();
    return ( $path, $query_params_str, $uri_authority, $uri_obj );
}

#TODO: replace this immediately with a secure solution that allows for only
#  app and mysql access
sub secure_file {
    my ( $self, $file ) = @_;
    if (   !exists( $self->{'handles'}->{$file} )
        || !$self->{'handles'}->{$file}->{'filename'} )
    {
        return 1;
    }
    my $filename = $self->{'handles'}->{$file}->{'filename'};
    my $mode     = 0777;
    return chmod( $mode, $filename );
}

# Create visitorid, $vidm contains the visitorid method
sub get_visitorid {
    my ($self, $line) = @_;
    my $vidm = $self->get_config_value('vidm') || 'VIDM_IPADDRESS';

    my $id_string = '';
    if ( $vidm eq 'VIDM_IPADDRESS' ) {
        $id_string = $line->{'host'};
    }
    elsif ( $vidm eq 'VIDM_IPPLUSAGENT' ) {
        $id_string = $line->{'host'} . ':' . $line->{'agent'};
    }
    elsif ( $vidm eq 'VIDM_COOKIE' ) {
        $line->{'cookie'} =~ m/Logaholic_VID=([^;]*);/;
        if ( defined $1 ) {
            $id_string = $1;
        }
        else {
            $id_string = $line->{'host'} . ':' . $line->{'agent'};
        }
    }
    return md5_hex($id_string);
}
1;
