#!/usr/bin/perl

use strict;
use warnings;

use lib ('../');

use Test::More tests => 30;

use Logaholic::IO::File;
use Logaholic::Log::Domlog;
use Logaholic::Preparse;
use URI::Encode ();

use Data::Dumper;

# stub profile
my $stub_profile_config = {};
{
    no warnings 'redefine';
    *Logaholic::Preparse::get_profile_config =
      sub { return $stub_profile_config; };
}

{
    my $logformat_regex = Logaholic::Log::Domlog::get_logformat_regex(
        'ApacheCombinedCookieLogParser');
    my $line =
qq(9.9.9.9 - - [16/Nov/2012:03:07:48 +0100] "GET /favicon.ico HTTP/1.1" 200 2550 "http://www.fqdn.tld/app/product_info.php/products_id/49" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000");

    like( $line, $logformat_regex,
'will properly match regex against regular line with referrer + agents + cookie'
    );
    my $nefarious_line =
qq(9.9.9.9 - - [13/Aug/2012:22:15:59 +0200] "GET /app/product_info.php?products_id=35'\" HTTP/1.0" 200 37372 "-" "-" "-");
    unlike( $nefarious_line, $logformat_regex,
'will properly not match regex against nefarious line with referrer + agents'
    );
}

my $lines = [
    {
        'test_txt' => 'equivalent domain in referrer; urlparamfilter exclude',
        'raw' =>
qq(logaholic.com - - [16/Nov/2012:03:07:48 +0100] "GET /admin.php?key1=val1&key2=val2 HTTP/1.1" 200 2550 "http://www.fqdn.tld/app/index.php?q=refq&foo=bar&bar=baz&key1=val1" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '2550',
            'status'    => '200',
            'time'      => '03:07:48',
            'date'      => '16/Nov/2012',
            'refparams' => 'bar=baz&q=refq&foo=bar',
            'referrer' => 'http://www.fqdn.tld/app/index.php',
            'datetime' => {
                'sec' => 48,
                'hr'  => '03',
                'yr'  => '2012',
                'min' => '07',
                'mon' => 10,
                'day' => '16',
            },
            'keywords'  => 'refq',
            'agent_md5' => '59b619f67063db72cd82f895d2804412',
            'url'       => '/admin.php',
            'method'    => 'GET',
            'gmt' => 1353031668,
            'country'   => 'NL',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              'NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000',
            'identuser' => '-',
            'host'      => 'logaholic.com',
            'urlparams' => '?key2=val2',
            'agent' =>
'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7',
            'crawl' => 0,
        },
        'profile' => {
            'urlparamfilter'     => 'key1',
            'urlparamfiltermode' => 'exclude',
            'equivdomains'       => 'fqdn.tld,foo.net',

        },
        'session' => 1,
    },
    {
        'test_txt' =>
          'non-equivalent domain in referrer; urlparamfilter exclude',
        'raw' =>
qq(logaholic.com - - [16/Nov/2012:03:08:48 +0100] "GET /admin.php?key1=val1&key2=val2 HTTP/1.1" 200 2550 "http://www.foo.tld/app/index.php?q=refq&foo=bar&bar=baz&key1=val1" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '2550',
            'status'    => '200',
            'time'      => '03:08:48',
            'date'      => '16/Nov/2012',
            'refparams' => 'bar=baz&q=refq&foo=bar',
            'referrer' => 'http://www.foo.tld/app/index.php',
            'datetime' => {
                'sec' => 48,
                'hr'  => '03',
                'yr'  => '2012',
                'min' => '08',
                'mon' => 10,
                'day' => '16',
            },
            'keywords'  => 'refq',
            'agent_md5' => '59b619f67063db72cd82f895d2804412',
            'url'       => '/admin.php',
            'method'    => 'GET',
            'gmt' => 1353031728,
            'country'   => 'NL',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              'NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000',
            'identuser' => '-',
            'host'      => 'logaholic.com',
            'urlparams' => '?key2=val2',
            'agent' =>
'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7',
            'crawl' => 0,
        },
        'profile' => {
            'urlparamfilter'     => 'key1',
            'urlparamfiltermode' => 'exclude',
            'equivdomains'       => 'fqdn.tld,foo.net',

        },
        'session' => 1,
    },
    {
        'test_txt' =>
          'non-equivalent domain in referrer; urlparamfilter include',
        'raw' =>
qq(logaholic.com - - [16/Nov/2012:05:08:48 +0100] "GET /admin.php?key1=val1&key2=val2 HTTP/1.1" 200 2550 "http://www.foo.tld/app/index.php?q=refq&foo=bar&bar=baz&key1=val1" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '2550',
            'status'    => '200',
            'time'      => '05:08:48',
            'date'      => '16/Nov/2012',
            'refparams' => 'key1=val1',
            'referrer' => 'http://www.foo.tld/app/index.php',
            'datetime' => {
                'sec' => 48,
                'hr'  => '05',
                'yr'  => '2012',
                'min' => '08',
                'mon' => 10,
                'day' => '16',
            },
            'keywords'  => '',
            'agent_md5' => '59b619f67063db72cd82f895d2804412',
            'url'       => '/admin.php',
            'method'    => 'GET',
            'gmt' => 1353038928,
            'country'   => 'NL',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              'NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000',
            'identuser' => '-',
            'host'      => 'logaholic.com',
            'urlparams' => '?key1=val1',
            'agent' =>
'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7',
            'crawl' => 0,
        },
        'profile' => {
            'urlparamfilter'     => 'key1',
            'urlparamfiltermode' => 'include',
            'equivdomains'       => 'fqdn.tld,foo.net',

        },
        'session' => 2,
    },
    {
        'test_txt' =>
'non-equivalent domain in referrer; urlparamfilter include; dynamic page hit',
        'raw' =>
qq(213.206.113.136 - - [16/Nov/2012:05:08:48 +0100] "GET /index.php?page=1&key1=val1&key2=val2 HTTP/1.1" 200 2550 "http://www.foo.tld/app/index.php?q=refq&foo=bar&bar=baz&key1=val1" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '2550',
            'status'    => '200',
            'time'      => '05:08:48',
            'date'      => '16/Nov/2012',
            'refparams' => 'key1=val1',
            'referrer' => 'http://www.foo.tld/app/index.php',
            'datetime' => {
                'sec' => 48,
                'hr'  => '05',
                'yr'  => '2012',
                'min' => '08',
                'mon' => 10,
                'day' => '16',
            },
            'keywords'  => '',
            'agent_md5' => '59b619f67063db72cd82f895d2804412',
            'url'       => '/index.php?page=1',
            'method'    => 'GET',
            'gmt' => 1353038928,
            'country'   => 'NL',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              'NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000',
            'identuser' => '-',
            'host'      => '213.206.113.136',
            'urlparams' => '?key1=val1',
            'agent' =>
'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7',
            'crawl' => 0,
        },
        'profile' => {
            'urlparamfilter'     => 'key1',
            'urlparamfiltermode' => 'include',
            'equivdomains'       => 'fqdn.tld,foo.net',
            'importantparams'    => '/index.php::page, /page.php::baz',
        },
        'session' => 3,
    },
    {
        'test_txt' =>
'non-equivalent domain in referrer; urlparamfilter exclude; dynamic page hit',
        'raw' =>
qq(213.206.113.136 - - [16/Nov/2012:05:08:49 +0100] "GET /page.php?baz=1&key1=val1&key2=val2 HTTP/1.1" 200 2550 "http://www.foo.tld/app/index.php?q=refq&foo=bar&bar=baz&key1=val1" "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7" "NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '2550',
            'status'    => '200',
            'time'      => '05:08:49',
            'date'      => '16/Nov/2012',
            'refparams' => 'bar=baz&q=refq&foo=bar',
            'referrer' => 'http://www.foo.tld/app/index.php',
            'datetime' => {
                'sec' => 49,
                'hr'  => '05',
                'yr'  => '2012',
                'min' => '08',
                'mon' => 10,
                'day' => '16',
            },
            'keywords'  => 'refq',
            'agent_md5' => '59b619f67063db72cd82f895d2804412',
            'url'       => '/page.php?baz=1',
            'method'    => 'GET',
            'gmt' => 1353038929,
            'country'   => 'NL',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              'NewLogaholic_VID=55229429000; NewLogaholic_SESSION=49097730000',
            'identuser' => '-',
            'host'      => '213.206.113.136',
            'urlparams' => '?key2=val2',
            'agent' =>
'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7',
            'crawl' => 0,
        },
        'profile' => {
            'urlparamfilter'     => 'key1',
            'urlparamfiltermode' => 'exclude',
            'equivdomains'       => 'fqdn.tld,foo.net',
            'importantparams'    => '/index.php::page, /page.php::baz',
        },
        'session' => 3,
    },

];
{
    foreach my $line ( @{$lines} ) {
        my $txt = $line->{'test_txt'};
        note("Test Scenario: $txt");

        my $logformat_regex = Logaholic::Log::Domlog::get_logformat_regex(
            'ApacheCombinedCookieLogParser');

        $stub_profile_config = $line->{'profile'};

        my $parser = Logaholic::Preparse->new();
        $parser->load_config_from_filename();

        my $raw      = $line->{'raw'};
        my $expected = $line->{'parsed'};

        my $line_data = Logaholic::Log::Domlog::parse( $logformat_regex, $raw );
        ok( $parser->is_valid_line($line_data), 'line can be parsed' );

        $parser->add_refinements_to_line_data($line_data);

        is_deeply( $line_data, $expected, "line parsed proper with refinements" );

    }
}

{
    note("Test Scenario: Verify total stats for domlog");
    my $parser = Logaholic::Preparse->new();
    $parser->load_config_from_filename();
    my $total_lines = scalar @{$lines};
    my $i           = 0;
    foreach my $line ( @{$lines} ) {
        ++$i;
        note("Processing line $i of $total_lines");
        my $logformat_regex = Logaholic::Log::Domlog::get_logformat_regex(
            'ApacheCombinedCookieLogParser');

        $stub_profile_config = $line->{'profile'};

        my $raw      = $line->{'raw'};
        my $expected = $line->{'parsed'};

        my $line_data = Logaholic::Log::Domlog::parse( $logformat_regex, $raw );
        $parser->add_refinements_to_line_data($line_data);
        $parser->update_statistics($line_data);
        my $expect_session = $line->{'session'};
        is( $line_data->{'sessioncounter'},
            $expect_session, 'line has correct session injected' );
    }
    my $expect_visitors = {
        'logaholic.com'   => { 'last_seen' => 1353038928, },
        '213.206.113.136' => { 'last_seen' => 1353038929, }
    };

    my $expect_total_bytes = {
        '2012-11-16' => 12750,
      },
      my $expect_total_mb = '0.0121593475341797';
    my $expect_sessioncounter = 3;
    my $expect_total_hits     = 5;

    note("Total stats of all $total_lines lines");
    is_deeply( $parser->{'visitors'}, $expect_visitors,
        'calculated all visitors and last seen' );
    is_deeply( $parser->{'total_bytes'},
        $expect_total_bytes, 'calculated total bytes per date key' );
    is_deeply( $parser->{'total_mb'}, $expect_total_mb, 'calculated total mb' );
    is_deeply( $parser->{'sessioncounter'},
        $expect_sessioncounter, 'calculated session count' );
    is_deeply( $parser->{'total_hits'},
        $expect_total_hits, 'calculated total hits' );

    #    delete $parser->{'geo'}; print Dumper $parser;
}

my $more_lines = [
    {
        'test_txt' =>
'query with urlparam, and refparams',
        'raw' =>
qq(109.58.110.35 - - [30/Nov/2012:00:02:11 +0100] "GET /manual/LogaholicManual/TroubleshootingWamp HTTP/1.1" 200 22260 "http://www.google.se/url?sa=t&rct=j&q=&esrc=s&source=web&cd=1&ved=0CC0QFjAA&url=http%3A%2F%2Fwww.logaholic.com%2Fmanual%2FLogaholicManual%2FTroubleshootingWamp&ei=cOm3UKunFY_T4QSD44CYDw&usg=AFQjCNEFz3c7N2ZH9aLlCnEp50lxjdAq2A&sig2=PW6yk8NKeFNkvgBYIrS7Hg" "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.25 Safari/537.17" "-"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '22260',
            'status'    => '200',
            'time'      => '00:02:11',
            'date'      => '30/Nov/2012',
#            'refparams' => '?sa=t&rct=j&q=&esrc=s&source=web&cd=1&ved=0CC0QFjAA&url=http%3A%2F%2Fwww.logaholic.com%2Fmanual%2FLogaholicManual%2FTroubleshootingWamp&ei=cOm3UKunFY_T4QSD44CYDw&usg=AFQjCNEFz3c7N2ZH9aLlCnEp50lxjdAq2A&sig2=PW6yk8NKeFNkvgBYIrS7Hg',
            'refparams' => 'q=',
            'referrer' => 'http://www.google.se/url',
            'datetime' => {
                'sec' => 11,
                'hr'  => '00',
                'yr'  => '2012',
                'min' => '02',
                'mon' => 10,
                'day' => '30',
            },
            'keywords'  => '(not provided)',
            'agent_md5' => '019785f63dc6ba51cdd2494d1f80924c',
            'url'       => '/manual/LogaholicManual/TroubleshootingWamp',
            'method'    => 'GET',
            'gmt' => 1354230131,
            'country'   => 'SE',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' => '-',
            'identuser' => '-',
            'host'      => '109.58.110.35',
            'urlparams' => '',
            'agent' =>
'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.25 Safari/537.17',
            'crawl' => 0,
        },
        'profile' => {
            'confdomain' => 'www.logaholic.com',
            'equivdomains' => '://logaholic.com',
            'vidm' => 'VIDM_IPPLUSAGENT',
            'lastpos' => '0',
            'firstlogline' => '0',
            'skiptime' => '',
            'googleparams' => 'q|start|gclid|as_q|as_epq|as_oq|as_eq|as_sitesearch|as_rq|as_lq',
            'skipfiles' => '.gif|.jpg|.jpeg|.js|.css|.txt|.ico|.swf|.xml|.png|.dll|logaholic/',
            'skipips' => '',
            'importantparams' =>'',
            'urlparamfiltermode' => 'Exclude',
            'urlparamfilter' => '',
        },
        'session' => 3,
    },
    {
        'test_txt' =>
'query with urlparam, and refparams',
        'raw' =>
qq(188.251.103.248 - - [30/Nov/2012:00:05:49 +0100] "GET /livehelp/image.php?what=userstat&page=www--dot--logaholic--dot--com/%3Fgclid%3DCISjrJWl9bMCFSHHtAodrHMA3A&randu=8325&pageid=6780&department=2&cslhVISITOR=65f0574c017088b71638f86b160a7c52&title=Web%20Analytics%20Software&referer=googleads--dot--g--dot--doubleclick--dot--net/pagead/ads%3Fclient%3Dca-pub-2899295842381068%26output%3Dhtml%26h%3D600%26slotname%3D5042362973%26w%3D160%26lmt%3D1354230343%26flash%3D11--dot--5--dot--31%26url%3Dhttp%253A%252F%252Fwww--dot--crowdrank--dot--net%252FSexiest-Woman-Alive%26dt%3D13542&cslheg=1&serversession=1 HTTP/1.1" 200 64 "http://www.logaholic.com/?gclid=CISjrJWl9bMCFSHHtAodrHMA3A" "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11" "__utma=116698651.1170627232.1354230353.1354230353.1354230353.1; __utmb=116698651.1.10.1354230353; __utmc=116698651; __utmz=116698651.1354230353.1.1.utmgclid=CISjrJWl9bMCFSHHtAodrHMA3A|utmccn=(not%20set)|utmcmd=(not%20set); NewLogaholic_VID=88138507369; NewLogaholic_SESSION=26610654847"),
        'parsed' => {
            'protocol'  => 'HTTP/1.1',
            'bytes'     => '64',
            'status'    => '200',
            'time'      => '00:05:49',
            'date'      => '30/Nov/2012',
            'refparams' => 'gclid=CISjrJWl9bMCFSHHtAodrHMA3A',
            'referrer' => 'http://www.logaholic.com/',
            'datetime' => {
                'sec' => 49,
                'hr'  => '00',
                'yr'  => '2012',
                'min' => '05',
                'mon' => 10,
                'day' => '30',
            },
            'keywords'  => '',
            'agent_md5' => 'edc6adaac6a66d678c53ef5ba4bfdb12',
            'url'       => '/livehelp/image.php',
            'method'    => 'GET',
            'gmt' => 1354230349,
            'country'   => 'PT',
            'authuser'  => '-',
            'timezone'  => '+0100',
            'cookie' =>
              '__utma=116698651.1170627232.1354230353.1354230353.1354230353.1; __utmb=116698651.1.10.1354230353; __utmc=116698651; __utmz=116698651.1354230353.1.1.utmgclid=CISjrJWl9bMCFSHHtAodrHMA3A|utmccn=(not%20set)|utmcmd=(not%20set); NewLogaholic_VID=88138507369; NewLogaholic_SESSION=26610654847',
            'identuser' => '-',
            'host'      => '188.251.103.248',
            'urlparams' => '?what=userstat&page=www--dot--logaholic--dot--com/%3Fgclid%3DCISjrJWl9bMCFSHHtAodrHMA3A&randu=8325&pageid=6780&department=2&cslhVISITOR=65f0574c017088b71638f86b160a7c52&title=Web%20Analytics%20Software&referer=googleads--dot--g--dot--doubleclick--dot--net/pagead/ads%3Fclient%3Dca-pub-2899295842381068%26output%3Dhtml%26h%3D600%26slotname%3D5042362973%26w%3D160%26lmt%3D1354230343%26flash%3D11--dot--5--dot--31%26url%3Dhttp%253A%252F%252Fwww--dot--crowdrank--dot--net%252FSexiest-Woman-Alive%26dt%3D13542&cslheg=1&serversession=1',
            'agent' =>
'Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11',
            'crawl' => 0,
        },
        'profile' => {
            'confdomain' => 'www.logaholic.com',
            'equivdomains' => '://logaholic.com',
            'vidm' => 'VIDM_IPPLUSAGENT',
            'lastpos' => '0',
            'firstlogline' => '0',
            'skiptime' => '',
            'googleparams' => 'q|start|gclid|as_q|as_epq|as_oq|as_eq|as_sitesearch|as_rq|as_lq',
            'skipfiles' => '.gif|.jpg|.jpeg|.js|.css|.txt|.ico|.swf|.xml|.png|.dll|logaholic/',
            'skipips' => '',
            'importantparams' =>'',
            'urlparamfiltermode' => 'Exclude',
            'urlparamfilter' => '',
        },
        'session' => 3,
    },

];


{
    foreach my $line ( @{$more_lines} ) {
        my $txt = $line->{'test_txt'};
        note("Test Scenario: $txt");

        my $logformat_regex = Logaholic::Log::Domlog::get_logformat_regex(
            'ApacheCombinedCookieLogParser');

        $stub_profile_config = $line->{'profile'};

        my $parser = Logaholic::Preparse->new();
        $parser->load_config_from_filename();

        my $raw      = $line->{'raw'};
        my $expected = $line->{'parsed'};

        my $line_data = Logaholic::Log::Domlog::parse( $logformat_regex, $raw );
        ok( $parser->is_valid_line($line_data), 'line can be parsed' );

        $parser->add_refinements_to_line_data($line_data);
        
        my @dynamic_items = qw(refparams urlparams);
        foreach my $item ( @dynamic_items ) {
            my $actual = make_hash_from_query($line_data->{$item});
            my $expect = make_hash_from_query($expected->{$item});
            is_deeply($actual, $expect, "dynamic item $item contains all elements");
            delete($line_data->{$item});
            delete($expected->{$item});
        }

        is_deeply( $line_data, $expected, "line parsed proper with refinements" );

    }
}

sub make_hash_from_query {
    my ($query) = @_;
    my $q_hr = {};
    return $q_hr if ! $query;
    $query =~ s/^\?//;
    my @qs = split('&', $query);
    foreach my $item ( @qs) {
        my ($key, $value) = split('=', $item);
        # this has to be done to conform with URI::QueryParms and URI behavior
        $value =~ s/\+/ /g;
        $value = URI::Encode::uri_encode($value, {encode_reserved => 1, double_encode => 0});
        
        $q_hr->{$key} = $value;
    }
    return $q_hr;
}