#!/usr/bin/perl

use strict;
use warnings;

use lib ('../');
use File::Temp;
use FindBin;
use Cwd qw(abs_path);
use JSON::Syck;

BEGIN {
    my $component_import_dir = abs_path( $FindBin::Bin . '/../../' );
    require "$component_import_dir/preparse.pl";
}

use Test::More tests => 4;

use Logaholic::Preparse;
use Logaholic::Log::Domlog;

my $test_data_dir = abs_path( $FindBin::Bin . '/data' );

my $cli_args = {
    'profile_config' => '',
    'schema_version' => 'v1',
    'output'         => 'JSON',
};

# Sub for creating temporary profile config that points to temporary input and
#  output files. Temp input is filled with lines from a sample log
sub make_domlog_setup {
    my ( $config, $inputfile, $outputfile ) =
      ( File::Temp->new(), File::Temp->new(), File::Temp->new(), );
    my ( $i_file, $o_file ) = (

        #$test_data_dir .'/test.domain.domlog',
        $inputfile->filename(),
        $outputfile->filename()
    );
    close($outputfile);

    my $data = <<CONFIG;
inputfile=$i_file
outputfile=$o_file
confdomain=www.opensoft.nl
KPI=/cdshop/checkout_shipping.php
skipfiles=.gif, .jpg, .js, .css, .txt, .ico, .swf, .xml, .png, .dll, logaholic/, admin/, cdsadmin/
urlparamfilter=osCsid
logformat=ApacheCombinedCookieLogParser
CONFIG
    print {$config} $data;
    close($config);

    open( my $domlog_fh, '<', $test_data_dir . '/test.domain.domlog' )
      || die $!;
    while ( my $line = readline($domlog_fh) ) {
        chomp $line if $line;
        print {$inputfile} "$line\n" if $line;
    }
    close($domlog_fh);
    close($inputfile);

    return ( $config, $inputfile, $outputfile );
}

note('get_output_string()');
{
    my $data = {
        'k1' => 'v1',
        'k2' => 'v2',
        'k3' => { 'kk1' => 'vv1', 'kk2', 'kk2' },
    };
    my $expect = JSON::Syck::Dump($data);
    my $actual =
      Logaholic::script::preparse->get_output_string( $data, 'JSON' );
    $actual =~ s/\n|\t//g;
    is( $actual, $expect, 'Can product JSON output string' );
}

{
    my $data = {
        'k1' => 'v1',
        'k2' => 'v2',
        'k3' => { 'kk1' => 'vv1', 'kk2' => 'vv2' },
    };
    my $expect = "k1:v1\nk2:v2\nk3:kk1=vv1\nk3:kk2=vv2\n";

    my $actual = Logaholic::script::preparse->get_output_string($data);
    is( $actual, $expect, 'Can product simple output string' );
}

note('Parse test.domain.domlog');
{
    my ( $config, $inputfile, $outputfile ) = make_domlog_setup();
    $cli_args->{'profile_config'} = $config;

    my $actual = Logaholic::script::preparse->script($cli_args);
    my $expect = {
        'bandwidth' => {
            '2012-07-20-04' => 456697,
            '2012-07-20-06' => 5081076,
            '2012-07-20-05' => 4600634
        },
        'lastlogpos' => 91769,
        'firstlogline' =>
'66.249.66.24 - - [20/Jul/2012:04:55:49 +0200] "GET /cdshop/advanced_search_result.php?keywords=uw&manufacturers_id=28&sort=1a&language=nl&osCsid=0cbra1hfv7i5lalm4oovh4qt21 HTTP/1.1" 200 31073 "-" "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)" "-"'
          . "\n"
    };
    is_deeply( $actual, $expect,
        "Can parse domlog based on profile and get proper stats" );

    my $parsed_stub = $test_data_dir . '/test.domain.domlog.parsed';
    my $diff_output = `diff -u $parsed_stub $outputfile`;
    is( $diff_output, '',
        "Parsed domlog generates expected outputfile contents" );
        open(my $fh, '>', '/tmp/test.domain.domlog.diff') || die $!;
        print ${fh} $diff_output;
}
